==> mls-0.1.tar.gz: mls-0.1/src/mcd_cur.cpp <==
#include <string>
#include <sys/stat.h>
#include <dirent.h>
#include "mcd.h"
#include "StringTokenizer.h"
#include "conf.h"

using namespace std;
using namespace MLS;

void Mcd::AddDirectory( const std::string &_dir )
{
	// 디렉토리가 있는지 맞는지 확인한다..
	struct stat stat_bufs;
	
	if (lstat(_dir.c_str(), &stat_bufs) == -1) return;	
	if (!S_ISDIR (stat_bufs.st_mode)) return;	
	if (!root) return;
	if (_dir.substr(0, root->name.size()) != root->name) return;
	
	Mcd::dir *p = root;	
	
	StringTokenizer st(_dir.substr(root->name.size()), "/"); // /을 중심으로 계속 끊어줌
	
	while(st.Next())
	{
		const string &token = st.Get();
		pDirIterator i;
		
		for (i = p->node.begin(); i!=p->node.end() ;i++)
			if ((*i)->name == token) break;
		
		if (i == p->node.end())
		{// 못찾았을 경우
			Mcd::dir *d = new Mcd::dir(token, p, 2);
			
			p->node.push_back(d);			
			if (conf.McdSort) sort(p->node.begin(), p->node.end(), mcdsort());
			
			// 디렉토리 많아질 수록 느려짐
			setOrder();		
			
			scan(d, false);			
			
			p = d;
		}
		else
		{// 찾았을 경우
			p = (*i);			
		}		
	}	
}

void 
Mcd::setCur(const string &str)
{
	//               01234567890123456789
	// root->name = "/sdfsd/sdfsdsf/"
	//              "/sdfsd/sdfsdsf/p/3/"
	
	if (str.empty())
	{
		cur = order.begin();
		return;				
	}
	
	if (   str[0] != '/'                // 절대경로가 아닐때
		|| str==root->name				// 루트일때
		|| str.size() < root->name.size()  // 루트사이즈보다 작을때
		|| str.substr(0, root->name.size()) != root->name)
	{
		
		cur = order.begin();
		return;
	}
	
	// /
	// 012345678901234
	// /home/ioklo/ppp
	StringTokenizer st(str.substr(root->name.size()), "/");	
	
	pDirIterator i;	
	dir *node = root;
	
	while (st.Next())
	{
		const string &t = st.Get();
		
		// 자식들 중에 있는거 찾아낸다.
		for (i = node->node.begin(); i!=node->node.end() ;i++)
			if ((*i)->name == t) break;		
		
		if (i == node->node.end()) {cur = order.begin(); return;}
		
		node = (*i);
	}
		
	cur = order.begin() + (*i)->index;}

void 
Mcd::up()
{
	pDirIterator i;
	
	if ((*cur)->parent)
	{		
		if ((*cur)->depth != (*(cur-1))->depth )
		{
			pDirIterator t;
			for (t = cur-1 ; t!=order.begin()-1; t--)
				if ((*t)->depth == (*cur)->depth) break;
				
			if (t != order.begin()-1) cur = t;
			else left();
		}
		else cur--;
	}
	else left();	
}

void 
Mcd::down()
{
	// 아래부분에 뭔가 있을 경우
	if ((*cur)->parent)
	{
		if (cur+1 == order.end() || (*cur)->depth != (*(cur+1))->depth )
		{
			pDirIterator t;
			for (t= cur+1 ; t!=order.end(); t++)
				if ((*t)->depth == (*cur)->depth) break;
				
			if (t != order.end()) cur = t;
		}
		else cur++;
	}
}

void
Mcd::left()
{
	
	if (!(*cur)->node.empty())
	{
		string p = (*cur)->path();
		destroy(*cur);
		setCur(p);
	}	 
		 
	else if ((*cur)->parent) cur=order.begin()+(*cur)->parent->index;
	
	
}

void
Mcd::right()
{
	if (!(*cur)->node.empty()) cur=order.begin()+(*cur)->node.front()->index;
	else 
	{
		string p = (*cur)->path();
		scan(*cur, false);
		setCur(p);
	//	if (cur+1 != order.end()) cur++;
	}
}

==> mls-0.2.tar.gz: mls-0.2/src/mcd_cur.cpp <==
///	\file	mcd_cur.cpp
///	\brief	mcd_cur.cpp
#include <string>
#include <sys/stat.h>
#include <dirent.h>
#include <algorithm>
#include "mcd.h"
#include "StringTokenizer.h"
#include "conf.h"

using namespace std;
using namespace MLS;

///	\brief	directory을 mcd에 추가한다.
///	\param	_dir	추가할 directory 명
void Mcd::AddDirectory( const std::string &_dir )
{
	// 디렉토리가 있는지 맞는지 확인한다..
	struct stat stat_bufs;
	
	if (lstat(_dir.c_str(), &stat_bufs) == -1) return;	
	if (!S_ISDIR (stat_bufs.st_mode)) return;	
	if (!root) return;
	if (_dir.substr(0, root->name.size()) != root->name) return;
	
	Mcd::dir *p = root;	
	
	StringTokenizer st(_dir.substr(root->name.size()), "/"); // /을 중심으로 계속 끊어줌
	
	while(st.Next())
	{
		const string &token = st.Get();
		pDirIterator i;
		
		for (i = p->node.begin(); i!=p->node.end() ;i++)
			if ((*i)->name == token) break;
		
		if (i == p->node.end())
		{// 못찾았을 경우
			Mcd::dir *d = new Mcd::dir(token, p, 2);
			
			p->node.push_back(d);		

			if (conf.McdSort) sort(p->node.begin(), p->node.end(), mcdsort());
			
			// 디렉토리 많아질 수록 느려짐
			setOrder();		
			
			scan(d, false);			
			
			p = d;
		}
		else
		{// 찾았을 경우
			p = (*i);			
		}		
	}	
}

///	\brief	str이 가르키는 directory로 현재 위치를 이동시킨다.
///	\param	str		현재 directory로 위치시키고자하는 directory명
void 
Mcd::setCur(const string &str)
{
	//               01234567890123456789
	// root->name = "/sdfsd/sdfsdsf/"
	//              "/sdfsd/sdfsdsf/p/3/"
	
	if (str.empty())
	{
		cur = order.begin();
		return;				
	}
	
	if (   str[0] != '/'                // 절대경로가 아닐때
		|| str==root->name				// 루트일때
		|| str.size() < root->name.size()  // 루트사이즈보다 작을때
		|| str.substr(0, root->name.size()) != root->name)
	{
		
		cur = order.begin();
		return;
	}
	
	// /
	// 012345678901234
	// /home/ioklo/ppp
	StringTokenizer st(str.substr(root->name.size()), "/");	
	
	pDirIterator i;	
	dir *node = root;
	
	while (st.Next())
	{
		const string &t = st.Get();
		
		// 자식들 중에 있는거 찾아낸다.
		for (i = node->node.begin(); i!=node->node.end() ;i++)
			if ((*i)->name == t) break;		
		
		if (i == node->node.end()) {cur = order.begin(); return;}
		
		node = (*i);
	}
		
	cur = order.begin() + (*i)->index;
}

///	\brief	mcd상에서 위 directory로 이동한다.
void 
Mcd::up()
{
	pDirIterator i;
	
	if ((*cur)->parent)
	{		
		if ((*cur)->depth != (*(cur-1))->depth )
		{
			pDirIterator t;
			for (t = cur-1 ; t!=order.begin()-1; t--)
				if ((*t)->depth == (*cur)->depth) break;
				
			if (t != order.begin()-1) cur = t;
			else left();
		}
		else cur--;
	}
	else left();	
}

///	\brief	mcd상에서 아래 directory로 이동한다.
void 
Mcd::down()
{
	// 아래부분에 뭔가 있을 경우
	if ((*cur)->parent)
	{
		if (cur+1 == order.end() || (*cur)->depth != (*(cur+1))->depth )
		{
			pDirIterator t;
			for (t= cur+1 ; t!=order.end(); t++)
				if ((*t)->depth == (*cur)->depth) break;
				
			if (t != order.end()) cur = t;
		}
		else cur++;
	}
}

///	\brief	mcd상에서 왼쪽(상위 directory)로 이동한다.
void
Mcd::left()
{
	if ((*cur)->parent) cur=order.begin()+(*cur)->parent->index;
}

///	\brief	mcd상에서 오른쪽(하위 directory)로 이동한다.
void
Mcd::right()
{
	if (!(*cur)->node.empty()) cur=order.begin()+(*cur)->node.front()->index;
	else 
	{
		string p = (*cur)->path();
		scan(*cur, false);
		setCur(p);
		if (!(*cur)->node.empty() && (*cur)->depth - sCol == 5) sCol = sCol + 1;
		
		
	//	if (cur+1 != order.end()) cur++;
	}
}
